from eppy import modeleditor
from eppy.modeleditor import IDF
import os

iddfile = r"C:\EnergyPlusV9-4-0\Energy+.idd"
# fname1 = r"D:\OneDrive - UNIVERSIDAD DE SEVILLA\Papers OneDrive\VPO_cadiz_parametrico\BA_V01.idf"

path = r'D:\Paper_VPO_cadiz_parametrico\input_⁯IDFs'
outputpath = r'D:\Paper_VPO_cadiz_parametrico\output_IDFs'

IDF.setiddname(iddfile)

file = path+'/BB_V01.idf'

idf1 = IDF(file)

# idf1.printidf()

if 'Cast Concrete_.1' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_cast_concrete = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'Cast Concrete_.1'][0]
    mat_cast_concrete.Roughness = 'Rough'
    mat_cast_concrete.Thickness = 0.1
    mat_cast_concrete.Conductivity = 1.13
    mat_cast_concrete.Density = 2000
    mat_cast_concrete.Specific_Heat = 1000
    mat_cast_concrete.Thermal_Absorptance = 0.9
    mat_cast_concrete.Solar_Absorptance = 0.6
    mat_cast_concrete.Visible_Absorptance = 0.6
else:
    idf1.newidfobject('MATERIAL',
        Name='Cast Concrete_.1',
        Roughness='Rough',
        Thickness=0.1,
        Conductivity=1.13,
        Density=2000,
        Specific_Heat=1000,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.6,
        Visible_Absorptance=0.6
        )

if 'O SO1_.16' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_S01 = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'O SO1_.16'][0]
    mat_S01.Roughness = 'Rough'
    mat_S01.Thickness = 0.16
    mat_S01.Conductivity = 0.01679
    mat_S01.Density = 1700
    mat_S01.Specific_Heat = 1000
    mat_S01.Thermal_Absorptance = 0.9
    mat_S01.Solar_Absorptance = 0.7
    mat_S01.Visible_Absorptance = 0.7
else:
    idf1.newidfobject('MATERIAL',
        Name='O SO1_.16',
        Roughness='Rough',
        Thickness=0.16,
        Conductivity=0.01679,
        Density=1700,
        Specific_Heat=1000,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.7,
        Visible_Absorptance=0.7
        )

if 'Floor/Roof Screed_.O7' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_screed = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'Floor/Roof Screed_.O7'][0]
    mat_screed.Roughness = 'Rough'
    mat_screed.Thickness = 0.07
    mat_screed.Conductivity = 0.41
    mat_screed.Density = 1200
    mat_screed.Specific_Heat = 840
    mat_screed.Thermal_Absorptance = 0.9
    mat_screed.Solar_Absorptance = 0.73
    mat_screed.Visible_Absorptance = 0.73
else:
    idf1.newidfobject('MATERIAL',
        Name='Floor/Roof Screed_.O7',
        Roughness='Rough',
        Thickness=0.07,
        Conductivity=0.41,
        Density=1200,
        Specific_Heat=840,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.73,
        Visible_Absorptance=0.73
        )


if 'OO SO1' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
    cons_S01 = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO SO1'][0]
    cons_S01.Name = 'OO SO1'
    cons_S01.Outside_Layer = 'Cast Concrete_.1'
    cons_S01.Layer_2 = 'O SO1_.16'
    cons_S01.Layer_3 = 'Floor/Roof Screed_.O7'
else:
    idf1.newidfobject(
        'CONSTRUCTION',
        Name='OO SO1',
        Outside_Layer='Cast Concrete_.1',
        Layer_2='O SO1_.16',
        Layer_3='Floor/Roof Screed_.O7'
        )

if 'OO SO1_Rev' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
    cons_S01_rev = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO SO1_Rev'][0]
    cons_S01_rev.Name = 'OO SO1_Rev'
    cons_S01_rev.Outside_Layer = 'Floor/Roof Screed_.O7'
    cons_S01_rev.Layer_2 = 'O SO1_.16'
    cons_S01_rev.Layer_3 = 'Cast Concrete_.1'
else:
    idf1.newidfobject(
        'CONSTRUCTION',
        Name='OO SO1_Rev',
        Outside_Layer='Floor/Roof Screed_.O7',
        Layer_2='O SO1_.16',
        Layer_3='Cast Concrete_.1'
        )









if 'OOO CTE Ladrillo perforado_.O5' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_perfbrick = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'OOO CTE Ladrillo perforado_.O5'][0]
    mat_perfbrick.Roughness = 'Rough'
    mat_perfbrick.Thickness = 0.05
    mat_perfbrick.Conductivity = 0.35
    mat_perfbrick.Density = 780
    mat_perfbrick.Specific_Heat = 1000
    mat_perfbrick.Thermal_Absorptance = 0.9
    mat_perfbrick.Solar_Absorptance = 0.7
    mat_perfbrick.Visible_Absorptance = 0.7
else:
    idf1.newidfobject(
        'MATERIAL',
        Name='OOO CTE Ladrillo perforado_.O5',
        Roughness='Rough',
        Thickness=0.05,
        Conductivity=0.35,
        Density=780,
        Specific_Heat=1000,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.7,
        Visible_Absorptance=0.7
    )

if 'O FO1_.125' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_F01 = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'O FO1_.125'][0]
    mat_F01.Roughness = 'Rough'
    mat_F01.Thickness = 0.125
    mat_F01.Conductivity = 0.01292
    mat_F01.Density = 1700
    mat_F01.Specific_Heat = 1000
    mat_F01.Thermal_Absorptance = 0.9
    mat_F01.Solar_Absorptance = 0.7
    mat_F01.Visible_Absorptance = 0.7
else:
    idf1.newidfobject(
        'MATERIAL',
        Name='O FO1_.125',
        Roughness='Rough',
        Thickness=0.125,
        Conductivity=0.01292,
        Density=1700,
        Specific_Heat=1000,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.7,
        Visible_Absorptance=0.7
    )

if 'OOO CTE Mortero_.O1' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_cementplaster = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'OOO CTE Mortero_.O1'][0]
    mat_cementplaster.Roughness = 'Rough'
    mat_cementplaster.Thickness = 0.01
    mat_cementplaster.Conductivity = 1
    mat_cementplaster.Density = 1700
    mat_cementplaster.Specific_Heat = 1000
    mat_cementplaster.Thermal_Absorptance = 0.9
    mat_cementplaster.Solar_Absorptance = 0.7
    mat_cementplaster.Visible_Absorptance = 0.7
else:
    idf1.newidfobject(
        'MATERIAL',
        Name='OOO CTE Mortero_.O1',
        Roughness='Rough',
        Thickness=0.01,
        Conductivity=1,
        Density=1700,
        Specific_Heat=1000,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.7,
        Visible_Absorptance=0.7
    )

if 'OO FO1' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
    cons_F01 = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO FO1'][0]
    cons_F01.Name = 'OO FO1'
    cons_F01.Outside_Layer = 'OOO CTE Ladrillo perforado_.O5'
    cons_F01.Layer_2 = 'O FO1_.125'
    cons_F01.Layer_3 = 'OOO CTE Mortero_.O1'
else:
    idf1.newidfobject(
        'CONSTRUCTION',
        Name='OO FO1',
        Outside_Layer='OOO CTE Ladrillo perforado_.O5',
        Layer_2='O FO1_.125',
        Layer_3='OOO CTE Mortero_.O1'
        )

if 'OO FO1_Rev' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
    cons_F01_rev = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO FO1_Rev'][0]
    cons_F01_rev.Name = 'OO FO1_Rev'
    cons_F01_rev.Outside_Layer = 'OOO CTE Ladrillo perforado_.O5'
    cons_F01_rev.Layer_2 = 'O FO1_.125'
    cons_F01_rev.Layer_3 = 'OOO CTE Mortero_.O1'
else:
    idf1.newidfobject(
        'CONSTRUCTION',
        Name='OO FO1_Rev',
        Outside_Layer='OOO CTE Mortero_.O1',
        Layer_2='O FO1_.125',
        Layer_3='OOO CTE Ladrillo perforado_.O5'
        )



if 'OOO CTE Arena_.O25' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_sand = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'OOO CTE Arena_.O25'][0]
    mat_sand.Roughness = 'Rough'
    mat_sand.Thickness = 0.025
    mat_sand.Conductivity = 2
    mat_sand.Density = 1700
    mat_sand.Specific_Heat = 910
    mat_sand.Thermal_Absorptance = 0.9
    mat_sand.Solar_Absorptance = 0.7
    mat_sand.Visible_Absorptance = 0.7
else:
    idf1.newidfobject(
        'MATERIAL',
        Name='OOO CTE Arena_.O25',
        Roughness='Rough',
        Thickness=0.025,
        Conductivity=2,
        Density=1700,
        Specific_Heat=910,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.7,
        Visible_Absorptance=0.7
    )

if 'O CO1_.2' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_C01 = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'O CO1_.2'][0]
    mat_C01.Roughness = 'Rough'
    mat_C01.Thickness = 0.2
    mat_C01.Conductivity = 0.03059
    mat_C01.Density = 1700
    mat_C01.Specific_Heat = 1000
    mat_C01.Thermal_Absorptance = 0.9
    mat_C01.Solar_Absorptance = 0.7
    mat_C01.Visible_Absorptance = 0.7
else:
    idf1.newidfobject(
        'MATERIAL',
        Name='O CO1_.2',
        Roughness='Rough',
        Thickness=0.2,
        Conductivity=0.03059,
        Density=1700,
        Specific_Heat=1000,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.7,
        Visible_Absorptance=0.7
    )

if 'Cast Concrete_.3' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
    mat_cast_concrete_thicker = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'Cast Concrete_.3'][0]
    mat_cast_concrete_thicker.Roughness = 'Rough'
    mat_cast_concrete_thicker.Thickness = 0.3
    mat_cast_concrete_thicker.Conductivity = 1.13
    mat_cast_concrete_thicker.Density = 2000
    mat_cast_concrete_thicker.Specific_Heat = 1000
    mat_cast_concrete_thicker.Thermal_Absorptance = 0.9
    mat_cast_concrete_thicker.Solar_Absorptance = 0.6
    mat_cast_concrete_thicker.Visible_Absorptance = 0.6
else:
    idf1.newidfobject(
        'MATERIAL',
        Name='Cast Concrete_.3',
        Roughness='Rough',
        Thickness=0.3,
        Conductivity=1.13,
        Density=2000,
        Specific_Heat=1000,
        Thermal_Absorptance=0.9,
        Solar_Absorptance=0.6,
        Visible_Absorptance=0.6
    )

if 'OO CO1' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
    cons_C01 = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO CO1'][0]
    cons_C01.Name = 'OO CO1'
    cons_C01.Outside_Layer = 'OOO CTE Arena_.O25'
    cons_C01.Layer_2 = 'O CO1_.2'
    cons_C01.Layer_3 = 'Cast Concrete_.3'
else:
    idf1.newidfobject(
        'CONSTRUCTION',
        Name='OO CO1',
        Outside_Layer='OOO CTE Arena_.O25',
        Layer_2='O CO1_.2',
        Layer_3='Cast Concrete_.3'
        )

if 'OO CO1_Rev' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
    cons_C01_rev = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO CO1_Rev'][0]
    cons_C01_rev.Name = 'OO CO1_Rev'
    cons_C01_rev.Outside_Layer = 'Cast Concrete_.3'
    cons_C01_rev.Layer_2 = 'O CO1_.2'
    cons_C01_rev.Layer_3 = 'OOO CTE Arena_.O25'
else:
    idf1.newidfobject(
        'CONSTRUCTION',
        Name='OO CO1_Rev',
        Outside_Layer='Cast Concrete_.3',
        Layer_2='O CO1_.2',
        Layer_3='OOO CTE Arena_.O25'
        )

floors = [i for i in idf1.idfobjects['BuildingSurface:Detailed']
         if i.Surface_Type=='Floor' and i.Outside_Boundary_Condition=='Ground']
# print(floors)
for i in floors:
    i.Construction_Name='OO SO1'

walls = [i for i in idf1.idfobjects['BuildingSurface:Detailed']
         if i.Surface_Type=='Wall' and i.Outside_Boundary_Condition=='Outdoors']
print(walls)
for i in walls:
    i.Construction_Name='OO FO1'

roofs = [i for i in idf1.idfobjects['BuildingSurface:Detailed']
         if i.Surface_Type=='Roof' and i.Outside_Boundary_Condition=='Outdoors']
# print(roofs)
for i in roofs:
    i.Construction_Name='OO CO1'


idf1.savecopy(file.split('.idf')[0]+'_amended.idf')


"""Example of script usage with arguments."""

from accim.sim import accis

scriptTypeList = ['vrf_mm']
outputsList = ['simplified', 'standard', 'timestep']
EPlist = ['22.2']

for i in scriptTypeList:
    for j in outputsList:
        for k in EPlist:
            accis.addAccis(
                ScriptType=i,
                Outputs=j,
                EnergyPlus_version=k,
                TempCtrl='temp',
                ComfStand=[1],
                CAT=[1],
                ComfMod=[1],
                HVACmode=[2],
                VentCtrl=[0],
                VSToffset=[0],
                MinOToffset=[50],
                MaxWindSpeed=[50],
                ASTtol_start=0.1,
                ASTtol_end_input=0.1,
                ASTtol_steps=0.1,
                NameSuffix=j
            )



"""
Run the function below to add the ACCIS.

This function transform fixed setpoint temperature
building energy models into adaptive setpoint temperature energy models
by adding the Adaptive Comfort Control Implementation Script (ACCIS)
"""

def addAccis(
        ScriptType: str = None,
        Outputs: str = None,
        EnergyPlus_version: str = None,
        TempCtrl: str = None,
        ComfStand: any = None,
        CAT: any = None,
        ComfMod: any = None,
        HVACmode: any = None,
        VentCtrl: any = None,
        VSToffset: any = [0],
        MinOToffset: any = [50],
        MaxWindSpeed: any = [50],
        ASTtol_start: float = 0.1,
        ASTtol_end_input: float = 0.1,
        ASTtol_steps: float = 0.1,
        NameSuffix: str = '',
        verboseMode: bool = True,
        confirmGen: bool = None):
    """
    Parameters
    :param ScriptType: The default is None.
    'vrf_ac' for VRFsystem with full air-conditionin mode, 
    'vrf_mm' for VRFsystem with mixed-mode, 
    'ex_ac' for ExistingHVAC only with full air-conditioning mode, 
    'ex_mm' for ExistingHVAC with mixed-mode.
    :param Outputs: The default is None. Can be 'standard', 'simplified' or 'timestep'.
    :param EnergyPlus_version: The default is None. Can be '9.1', '9.2', '9.3', '9.4', '9.5', '9.6', '22.1' or '22.2'.
    :param TempCtrl: The default is None. Can be 'temp' or 'pmv'.
    :param ComfStand: The default is None.
        '0 = ESP CTE;
        '1 = INT EN16798;
        '2 = INT ASHRAE55;
        '3 = JPN Rijal;
        '4 = CHN GBT50785 Cold;
        '5 = CHN GBT50785 HotMild;
        '6 = CHN Yang;
        '7 = IND IMAC C NV;
        '8 = IND IMAC C MM;
        '9 = IND IMAC R 7DRM;
        '10 = IND IMAC R 30DRM;
        '11 = IND Dhaka;
        '12 = ROM Udrea;
        '13 = AUS Williamson;
        '14 = AUS DeDear;
        '15 = BRA Rupp NV;
        '16 = BRA Rupp AC;
        '17 = MEX Oropeza Arid;
        '18 = MEX Oropeza DryTropic;
        '19 = MEX Oropeza Temperate;
        '20 = MEX Oropeza HumTropic;
    :param CAT: The default is None.
    (1 = CAT I; 2 = CAT II; 3 = CAT III; 80 = 80% ACCEPT; 85 = 85% ACCEPT; 90 = 90% ACCEPT)
    :param ComfMod: The default is None.
    (0 = Static;
    1 = Adaptive when applicable, otherwise relevant local static model;
    2 = Adaptive when applicable, otherwise relevant international static model
    3 = Adaptive when applicable, otherwise horizontal extention of adaptive setpoints)
    :param HVACmode: The default is None.
    (0 = Fully Air-conditioned; 1 = Naturally ventilated; 2 = Mixed Mode)
    :param VentCtrl: The default is None.
    (0 = Ventilates above neutral temperature; 1 = Ventilates above upper comfort limit)
    :param VSToffset: The default is 0. Please refer to documentation.
    :param MinOToffset: The default is 50. Please refer to documentation.
    :param MaxWindSpeed: The default is 50. Please refer to documentation.
    :param ASTtol_start: The default is 0.1. Please refer to documentation.
    :param ASTtol_end_input: The default is 0.1. Please refer to documentation.
    :param ASTtol_steps: The default is 0.1. Please refer to documentation.
    :param NameSuffix: The default is '' (an empty string). Please refer to documentation.
    :param verboseMode: True to print the process on screen. Default is True.
    :param confirmGen: True to skip confirmation of output IDF generation. Default is None.

    Exceptions
            DESCRIPTION. EnergyPlus version not supported.
        Only works for versions between EnergyPlus 9.1 (enter 9.1) and
        EnergyPlus 22.2 (enter 22.2).

    :return:
    Returns nothing. Output IDFs are generated in the source folder, where input IDFs were located.
    """
    import accim.sim.accim_Main as accim_Main
    from os import listdir, remove
    import accim

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not '[' in file])

    filelist = ([file.split('.idf')[0] for file in filelist])

    objArgsDef = (
        ScriptType is not None,
        Outputs is not None,
        EnergyPlus_version is not None,
        TempCtrl is not None,
    )

    fullScriptTypeList = ['vrf_ac',
                          'vrf_mm',
                          'ex_mm',
                          'ex_ac',
                          ]

    fullOutputsList = [
        'Standard',
        'standard',
        'Simplified',
        'simplified',
        'Timestep',
        'timestep'
    ]

    fullEPversionsList = [
        '9.1',
        '9.2',
        '9.3',
        '9.4',
        '9.5',
        '9.6',
        '22.1',
        '22.2'
    ]

    fullTempCtrllist = [
        'temperature',
        'temp',
        'pmv'
    ]

    if all(objArgsDef):
        pass
    else:
        ScriptType = input("Enter the ScriptType (\n"
                           "for VRFsystem with full air-conditioning mode: vrf_ac;\n"
                           "for VRFsystem with mixed-mode: vrf_mm;\n"
                           "for ExistingHVAC with mixed mode: ex_mm;\n"
                           "for ExistingHVAC with full air-conditioning mode: ex_ac\n"
                           "): ")
        while ScriptType not in fullScriptTypeList:
            ScriptType = input("    ScriptType was not correct. "
                               "    Enter the ScriptType (\n"
                               "    for VRFsystem with full air-conditioning mode: vrf_ac;\n"
                               "    for VRFsystem with mixed-mode: vrf_mm;\n"
                               "    for ExistingHVAC with mixed mode: ex_mm;\n"
                               "    for ExistingHVAC with full air-conditioning mode: ex_ac\n"
                               "    ): ")
        Outputs = input("Enter the Output (standard, simplified or timestep): ")
        while Outputs not in fullOutputsList:
            Outputs = input("   Output was not correct. "
                            "Please, enter the Output (standard, simplified or timestep): ")
        EnergyPlus_version = input("Enter the EnergyPlus version (9.1 to 22.2): ")
        while EnergyPlus_version not in fullEPversionsList:
            EnergyPlus_version = input("    EnergyPlus version was not correct. "
                                       "Please, enter the EnergyPlus version (9.1 to 22.2): ")
        TempCtrl = input('Enter the Temperature Control method (temperature or pmv): ')
        while TempCtrl not in fullTempCtrllist:
            TempCtrl = input("  Temperature Control method was not correct. "
                             "Please, enter the Temperature Control method (temperature or pmv): ")


    if verboseMode:
        print('Basic input data:')
        # print(f'accim version: {accim.__version__}')
        print('ScriptType is: '+ScriptType)
    if ScriptType not in fullScriptTypeList:
        print('Valid ScriptTypes: ')
        print(fullScriptTypeList)
        raise ValueError(ScriptType + " is not a valid ScriptType. "
                                      "You must choose a ScriptType from the list above.")
    if verboseMode:
        print('Outputs are: '+Outputs)
    if Outputs not in fullOutputsList:
        print('Valid Outputs: ')
        print(fullOutputsList)
        raise ValueError(Outputs + " is not a valid Output. "
                                   "You must choose a Output from the list above.")
    if verboseMode:
        print('EnergyPlus version is: '+EnergyPlus_version)
    if EnergyPlus_version not in fullEPversionsList:
        print('Valid EnergyPlus_version: ')
        print(fullEPversionsList)
        raise ValueError(EnergyPlus_version + " is not a valid EnergyPlus_version. "
                                              "You must choose a EnergyPlus_version"
                                              "from the list above.")
    if verboseMode:
        print('Temperature Control method is: '+TempCtrl)
    if TempCtrl not in fullTempCtrllist:
        print('Valid Temperature Control methods: ')
        print(fullTempCtrllist)
        raise ValueError(TempCtrl + " is not a valid Temperature Control method. "
                                              "You must choose a Temperature Control method"
                                              "from the list above.")

    notWorkingIDFs = []

    for file in filelist:
        if verboseMode:
            print('''\n=======================START OF GENERIC IDF FILE GENERATION PROCESS=======================\n''')
            print('Starting with file:')
            print(file)
        z = accim_Main.accimJob(
            filename_temp=file,
            ScriptType=ScriptType,
            EnergyPlus_version=EnergyPlus_version,
            TempCtrl=TempCtrl,
            verboseMode=verboseMode
        )

        if z.accimNotWorking is True:
            # raise KeyError(f'accim is not going to work with {file}')
            notWorkingIDFs.append(file)
            continue

        z.setComfFieldsPeople(EnergyPlus_version=EnergyPlus_version, TempCtrl=TempCtrl, verboseMode=verboseMode)

        if 'vrf' in ScriptType.lower():
            if TempCtrl.lower() == 'temperature' or TempCtrl.lower() == 'temp':
                z.addOpTempTherm(verboseMode=verboseMode)
            elif TempCtrl.lower() == 'pmv':
                z.setPMVsetpoint(verboseMode=verboseMode)
            z.addBaseSchedules(verboseMode=verboseMode)
            z.setAvailSchOn(verboseMode=verboseMode)
            z.addVRFsystemSch(verboseMode=verboseMode)
            z.addCurveObj(verboseMode=verboseMode)
            z.addDetHVACobj(EnergyPlus_version=EnergyPlus_version, verboseMode=verboseMode, TempCtrl=TempCtrl)
            if ScriptType.lower() == 'vrf_mm':
                z.checkVentIsOn(verboseMode=verboseMode)
            z.addForscriptSchVRFsystem(verboseMode=verboseMode)
        elif 'ex' in ScriptType.lower():
            # todo check if PMV can work with ex_ac
            z.addForscriptSchExistHVAC(verboseMode=verboseMode)

        z.addEMSProgramsBase(ScriptType=ScriptType, verboseMode=verboseMode)
        z.addEMSOutputVariableBase(ScriptType=ScriptType, verboseMode=verboseMode)
        z.addGlobVarList(ScriptType=ScriptType, verboseMode=verboseMode)
        z.addIntVarList(verboseMode=verboseMode)
        z.addEMSSensorsBase(ScriptType=ScriptType, verboseMode=verboseMode)
        z.addEMSActuatorsBase(ScriptType=ScriptType, verboseMode=verboseMode)

        if 'vrf' in ScriptType.lower():
            z.addEMSSensorsVRFsystem(ScriptType=ScriptType, verboseMode=verboseMode)
        elif ScriptType.lower() == 'ex_mm':
            z.addEMSSensorsExisHVAC(verboseMode=verboseMode)

        z.addEMSPCMBase(verboseMode=verboseMode)

        if Outputs.lower() == 'simplified':
            z.addSimplifiedOutputVariables(
                TempCtrl=TempCtrl,
                verboseMode=verboseMode
            )
        elif Outputs.lower() == 'standard':
            z.addOutputVariablesBase(
                ScriptType=ScriptType,
                TempCtrl=TempCtrl,
                verboseMode=verboseMode
            )

        if Outputs.lower() == 'timestep':
            z.addOutputVariablesTimestep(verboseMode=verboseMode)

        z.saveaccim(verboseMode=verboseMode)
        if verboseMode:
            print('Ending with file:')
            print(file)
            print('''\n=======================END OF GENERIC IDF FILE GENERATION PROCESS=======================\n''')

    if verboseMode:
        print('The following IDFs will not work, and therefore these will be deleted:')
    if len(notWorkingIDFs) > 0:
        if verboseMode:
            print(*notWorkingIDFs, sep="\n")
        filelist_pymod = ([file for file in listdir() if file.endswith('.idf')
                     and '_pymod' in file])

        for file in notWorkingIDFs:
            for i in filelist_pymod:
                if file in i:
                    remove(i)
    else:
        if verboseMode:
            print('None')

    if verboseMode:
        print('''\n=======================START OF OUTPUT IDF FILES GENERATION PROCESS=======================\n''')

    args_needed_mm = (
        ComfStand is not None,
        CAT is not None,
        ComfMod is not None,
        HVACmode is not None,
        VentCtrl is not None,
    )

    args_needed_ac = (
        ComfStand is not None,
        CAT is not None,
        ComfMod is not None,
    )
    if ScriptType.lower() == 'vrf_mm' or ScriptType.lower() == 'ex_mm':
        if all(args_needed_mm):
            z.genIDF(
                ScriptType=ScriptType,
                TempCtrl=TempCtrl,
                ComfStand=ComfStand,
                CAT=CAT,
                ComfMod=ComfMod,
                HVACmode=HVACmode,
                VentCtrl=VentCtrl,
                VSToffset=VSToffset,
                MinOToffset=MinOToffset,
                MaxWindSpeed=MaxWindSpeed,
                ASTtol_start=ASTtol_start,
                ASTtol_end_input=ASTtol_end_input,
                ASTtol_steps=ASTtol_steps,
                NameSuffix=NameSuffix,
                verboseMode=verboseMode,
                confirmGen=confirmGen
                )
        else:
            z.inputData(
                ScriptType=ScriptType,
            )
            z.genIDF(
                ScriptType=ScriptType,
                TempCtrl=TempCtrl,
            )
    elif ScriptType.lower() == 'ex_ac' or ScriptType.lower() == 'vrf_ac':
        if all(args_needed_ac):
            z.genIDF(
                ScriptType=ScriptType,
                TempCtrl=TempCtrl,
                ComfStand=ComfStand,
                CAT=CAT,
                ComfMod=ComfMod,
                HVACmode=[0],
                VentCtrl=[0],
                VSToffset=[0],
                MinOToffset=[0],
                MaxWindSpeed=[0],
                ASTtol_start=ASTtol_start,
                ASTtol_end_input=ASTtol_end_input,
                ASTtol_steps=ASTtol_steps,
                NameSuffix=NameSuffix,
                verboseMode=verboseMode,
                confirmGen=confirmGen
                )
        else:
            z.inputData(
                ScriptType=ScriptType,
            )
            z.genIDF(
                ScriptType=ScriptType,
                TempCtrl=TempCtrl,
            )

    if verboseMode:
        print('''\n=======================END OF OUTPUT IDF FILES GENERATION PROCESS=======================\n''')



from accim.sim import accis
accis.addAccis(
    ScriptType='ex_mm',
    Outputs='standard',
    EnergyPlus_version='9.5',
    TempCtrl='temp',
    ComfStand=[1],
    CAT=[1, 2, 3],
    ComfMod=[3],
    HVACmode=[2],
    VentCtrl=[0],
    confirmGen=True
)


"""Add EMS objects only for VRFsystem."""


def addEMSSensorsVRFsystem(self, ScriptType: str = None, verboseMode: bool = True):
    """Add EMS sensors for VRF system accim."""
    sensorlist = ([sensor.Name for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor']])
    for i in range(len(self.zonenames)):
        if self.zonenames[i] + '_CoolCoil' in sensorlist:
            if verboseMode:
                print('Not added - ' + self.zonenames[i] + '_CoolCoil Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.zonenames[i] + '_CoolCoil',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.zonenames_orig[i] + ' VRF Indoor Unit DX Cooling Coil',
                OutputVariable_or_OutputMeter_Name='Cooling Coil Total Cooling Rate'
            )
            if verboseMode:
                print('Added - ' + self.zonenames[i] + '_CoolCoil Sensor')
        #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.zonenames[i]+'_CoolCoil'])
        if self.zonenames[i] + '_HeatCoil' in sensorlist:
            if verboseMode:
                print('Not added - ' + self.zonenames[i] + '_HeatCoil Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.zonenames[i] + '_HeatCoil',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.zonenames_orig[i] + ' VRF Indoor Unit DX Heating Coil',
                OutputVariable_or_OutputMeter_Name='Heating Coil Heating Rate'
            )
            if verboseMode:
                print('Added - ' + self.zonenames[i] + '_HeatCoil Sensor')
    #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.zonenames[i]+'_HeatCoil'])

    if ScriptType == 'vrf_mm':
        for i in range(len(self.windownamelist)):
            if self.windownamelist[i]+'_CoolCoil' in sensorlist:
                if verboseMode:
                    print('Not added - '+self.windownamelist[i]+'_CoolCoil Sensor')
            else:
                self.idf1.newidfobject(
                    'EnergyManagementSystem:Sensor',
                    Name=self.windownamelist[i]+'_CoolCoil',
                    OutputVariable_or_OutputMeter_Index_Key_Name=self.windownamelist_orig_split[i][0]+' VRF Indoor Unit DX Cooling Coil',
                    OutputVariable_or_OutputMeter_Name='Cooling Coil Total Cooling Rate'
                    )
                if verboseMode:
                    print('Added - '+self.windownamelist[i]+'_CoolCoil Sensor')

            if self.windownamelist[i]+'_HeatCoil' in sensorlist:
                if verboseMode:
                    print('Not added - '+self.windownamelist[i]+'_HeatCoil Sensor')
            else:
                self.idf1.newidfobject(
                    'EnergyManagementSystem:Sensor',
                    Name=self.windownamelist[i]+'_HeatCoil',
                    OutputVariable_or_OutputMeter_Index_Key_Name=self.windownamelist_orig_split[i][0]+' VRF Indoor Unit DX Heating Coil',
                    OutputVariable_or_OutputMeter_Name='Heating Coil Heating Rate'
                    )
                if verboseMode:
                    print('Added - '+self.windownamelist[i]+'_HeatCoil Sensor')

    del sensorlist



def addEMSSensorsExisHVAC(self, verboseMode : bool = True):
    
    
    sensorlist = ([sensor.Name for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor']])
    for i in range(len(self.ExisHVAC)):
        for j in range(len(self.ExisHVAC[i][1])):
            if 'Cool' in self.ExisHVAC[i][1][j] or 'Cool' in self.HVACdict[self.ExisHVAC[i][0]]:
                if self.ExisHVAC[i][3][j] + '_CoolCoil' in sensorlist:
                    if verboseMode:
                        print('Not added - ' + self.ExisHVAC[i][3][j] + '_CoolCoil Sensor')
                else:
                    self.idf1.newidfobject(
                        'EnergyManagementSystem:Sensor',
                        Name=self.ExisHVAC[i][3][j] + '_CoolCoil',
                        OutputVariable_or_OutputMeter_Index_Key_Name=self.ExisHVAC[i][1][j],
                        OutputVariable_or_OutputMeter_Name=self.HVACdict[self.ExisHVAC[i][0]]
                    )
                    if verboseMode:
                        print('Added - ' + self.ExisHVAC[i][3][j] + '_CoolCoil Sensor')
                for k in range(len(self.ExisHVAC[i][4])):
                    if self.ExisHVAC[i][3][j] in self.ExisHVAC[i][4][k]:
                        if self.ExisHVAC[i][4][k] + '_CoolCoil' in sensorlist:
                            if verboseMode:
                                print('Not added - ' + self.ExisHVAC[i][4][k] + '_CoolCoil Sensor')
                        else:
                            self.idf1.newidfobject(
                                'EnergyManagementSystem:Sensor',
                                Name=self.ExisHVAC[i][4][k] + '_CoolCoil',
                                OutputVariable_or_OutputMeter_Index_Key_Name=self.ExisHVAC[i][1][j],
                                OutputVariable_or_OutputMeter_Name=self.HVACdict[self.ExisHVAC[i][0]]
                            )
                            if verboseMode:
                                print('Added - ' + self.ExisHVAC[i][4][k] + '_CoolCoil Sensor')
                #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.ExisHVAC[i][3][j]+'_CoolCoil'])
            if 'Heating' in self.ExisHVAC[i][1][j] or 'Heating' in self.HVACdict[self.ExisHVAC[i][0]]:
                if self.ExisHVAC[i][3][j] + '_HeatCoil' in sensorlist:
                    if verboseMode:
                        print('Not added - ' + self.ExisHVAC[i][3][j] + '_HeatCoil Sensor')
                else:
                    self.idf1.newidfobject(
                        'EnergyManagementSystem:Sensor',
                        Name=self.ExisHVAC[i][3][j] + '_HeatCoil',
                        OutputVariable_or_OutputMeter_Index_Key_Name=self.ExisHVAC[i][1][j],
                        OutputVariable_or_OutputMeter_Name=self.HVACdict[self.ExisHVAC[i][0]]
                    )
                    if verboseMode:
                        print('Added - ' + self.ExisHVAC[i][3][j] + '_HeatCoil Sensor')
                # probando
                for k in range(len(self.ExisHVAC[i][4])):
                    if self.ExisHVAC[i][3][j] in self.ExisHVAC[i][4][k]:
                        if self.ExisHVAC[i][4][k] + '_HeatCoil' in sensorlist:
                            if verboseMode:
                                print('Not added - ' + self.ExisHVAC[i][4][k] + '_HeatCoil Sensor')
                        else:
                            self.idf1.newidfobject(
                                'EnergyManagementSystem:Sensor',
                                Name=self.ExisHVAC[i][4][k] + '_HeatCoil',
                                OutputVariable_or_OutputMeter_Index_Key_Name=self.ExisHVAC[i][1][j],
                                OutputVariable_or_OutputMeter_Name=self.HVACdict[self.ExisHVAC[i][0]]
                            )
                            if verboseMode:
                                print('Added - ' + self.ExisHVAC[i][4][k] + '_HeatCoil Sensor')



from eppy import modeleditor
from eppy.modeleditor import IDF
import os

iddfile = r"C:\EnergyPlusV9-4-0\Energy+.idd"
# fname1 = r"D:\OneDrive - UNIVERSIDAD DE SEVILLA\Papers OneDrive\VPO_cadiz_parametrico\BA_V01.idf"

path = r'C:\Users\daniel.sanchez\Documents\Personal\VPO_parametrico\input_IDFs'
outputpath = r'C:\Users\daniel.sanchez\Documents\Personal\VPO_parametrico\output_IDFs'

filelist = [file.split('.idf')[0] for file in os.listdir(path) if file.endswith('.idf')]
print(filelist)

IDF.setiddname(iddfile)

# idf1.printidf()



facade_cond_list = [
    0.01292,
    0.02673,
    0.04152,
    0.05741,
    0.07453,
    0.09302,
    0.11305,
    0.13482,
    0.15858,
    0.18460,
    0.21323,
    0.24487,
    0.28003,
    0.31934,
    0.36357,
    0.41371,
    0.47103,
    0.53718,
    0.61438,
    0.70565,
]

facade_cond_list_full = [
    0.01292,
    0.02673,
    0.04152,
    0.05741,
    0.07453,
    0.09302,
    0.11305,
    0.13482,
    0.15858,
    0.18460,
    0.21323,
    0.24487,
    0.28003,
    0.31934,
    0.36357,
    0.41371,
    0.47103,
    0.53718,
    0.61438,
    0.70565,
    0.81522,
    0.94921,
    1.11681,
    1.33249,
    1.62037,
    2.02402,
    2.63085,
    3.64583,
    5.68946,
    11.93182
]

roof_cond_list = [
    0.03059,
    0.06240,
    0.09552,
    0.13001,
    0.16598,
    0.20350,
    0.24270,
    0.28369,
    0.32658,
    0.37152,
    0.41865,
    0.46814,
    0.52017,
    0.57495,
    0.63269,
    0.69364,
    0.75808,
    0.82632,
    0.89870,
    0.97561,
]

roof_cond_list_full = [
    0.03059,
    0.06240,
    0.09552,
    0.13001,
    0.16598,
    0.20350,
    0.24270,
    0.28369,
    0.32658,
    0.37152,
    0.41865,
    0.46814,
    0.52017,
    0.57495,
    0.63269,
    0.69364,
    0.75808,
    0.82632,
    0.89870,
    0.97561,
    1.05749,
    1.14484,
    1.23822,
    1.33829,
    1.44578,
    1.56156,
    1.68662,
    1.82213,
    1.96944,
    2.13018
]

floor_cond_list_full = [
    0.01679,
    0.03531,
    0.05586,
    0.07879,
    0.10452,
    0.13362,
    0.16678,
    0.20493,
    0.24927,
    0.30145,
    0.36375,
    0.43943,
    0.53333,
    0.65291,
    0.81038,
    1.02714,
    1.34445,
    1.85339,
    2.80267,
    5.19941,
    22.97796,
    -10.89783,
    -4.64513,
    -3.04410,
    -2.31123,
    -1.89099,
    -1.61850,
    -1.42750,
    -1.28618,
    -1.17739
]

floor_cond_list = [
    0.01679,
    0.03531,
    0.05586,
    0.07879,
    0.10452,
    0.13362,
    0.16678,
    0.20493,
    0.24927,
    0.30145,
    0.36375,
    0.43943,
    0.53333,
    0.65291,
    0.81038,
    1.02714,
    1.34445,
    1.85339,
    2.80267,
    5.19941,
]

for file in filelist:
    filename = file
    print(file)
    fname1 = path+'/'+filename + '.idf'
    idf1 = IDF(fname1)

    C01 = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'O CO1_.3'][0]
    C01.Name = 'O CO1_.2'
    C01.Thickness = 0.2

    slab = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'OOO Hormigon celular_.1'][0]
    slab.Name = 'Cast Concrete_.3'
    slab.Thickness = 0.3
    slab.Conductivity = 1.13
    slab.Density = 2000
    slab.Thermal_Absorptance = 0.9
    slab.Solar_Absorptance = 0.6
    slab.Visible_Absorptance = 0.6

    buildup = [cons for cons in idf1.idfobjects['CONSTRUCTION'] if cons.Name == 'OO CO1'][0]
    buildup.Layer_2 = 'O CO1_.2'
    buildup.Layer_3 = 'Cast Concrete_.3'

    buildup_rev = [cons for cons in idf1.idfobjects['CONSTRUCTION'] if cons.Name == 'OO CO1_Rev'][0]
    buildup_rev.Outside_Layer = 'Cast Concrete_.3'
    buildup_rev.Layer_2 = 'O CO1_.2'

    facade_mat = [mat
                  for mat
                  in idf1.idfobjects['MATERIAL']
                  if mat.Name == 'O FO1_.125'][0]
    # print(facade_mat)
    roof_mat = [mat
                for mat
                in idf1.idfobjects['MATERIAL']
                if mat.Name == 'O CO1_.2'][0]
    # print(roof_mat)
    try:
        floor_mat = [mat
                     for mat
                     in idf1.idfobjects['MATERIAL']
                     if mat.Name == 'O SO1_.16'][0]
        # print(floor_mat)
        therearefloors = True
    except IndexError:
        therearefloors = False
        # print(therearefloors)
    facade_cond = facade_mat.Conductivity
    # print(facade_cond)
    roof_cond = roof_mat.Conductivity
    # print(roof_cond)
    if therearefloors:
        floor_cond = floor_mat.Conductivity
        # print(floor_cond)
    # else:
    #     print('there are no floors')
    for i in range(len(facade_cond_list)):
        facade_mat.Conductivity = facade_cond_list[i]
        for j in range(len(roof_cond_list)):
            roof_mat.Conductivity = roof_cond_list[j]
            if therearefloors:
                floor_mat.Conductivity = floor_cond_list[j]
                if i+1 < 10:
                    if j+1 < 10:
                        outputname = (
                            filename
                            + '[F0'+repr(i+1)
                            + '[C0'+repr(j+1)
                            + '[S0'+repr(j+1)
                            + '.idf'
                        )
                    else:
                        outputname = (
                            filename
                            + '[F0'+repr(i+1)
                            + '[C'+repr(j+1)
                            + '[S'+repr(j+1)
                            + '.idf'
                        )
                else:
                    if j+1 < 10:
                        outputname = (
                            filename
                            + '[F'+repr(i+1)
                            + '[C0'+repr(j+1)
                            + '[S0'+repr(j+1)
                            + '.idf'
                        )
                    else:
                        outputname = (
                            filename
                            + '[F'+repr(i+1)
                            + '[C'+repr(j+1)
                            + '[S'+repr(j+1)
                            + '.idf'
                        )
            else:
                if i+1 < 10:
                    if j+1 < 10:
                        outputname = (
                            filename
                            + '[F0'+repr(i+1)
                            + '[C0'+repr(j+1)
                            + '[SXX'
                            + '.idf'
                        )
                    else:
                        outputname = (
                            filename
                            + '[F0'+repr(i+1)
                            + '[C'+repr(j+1)
                            + '[SXX'
                            + '.idf'
                        )
                else:
                    if j+1 < 10:
                        outputname = (
                            filename
                            + '[F'+repr(i+1)
                            + '[C0'+repr(j+1)
                            + '[SXX'
                            + '.idf'
                        )
                    else:
                        outputname = (
                            filename
                            + '[F'+repr(i+1)
                            + '[C'+repr(j+1)
                            + '[SXX'
                            + '.idf'
                        )
            idf1.savecopy(outputpath+'/'+outputname)
